from pydantic import (
    BaseModel,
    Field,
    ConfigDict,
)

from mikrotikapi.schemes.components import SrcAddressBlock, ToAddresses
from mikrotikapi.schemes.components.action_block import ActionBlock
from mikrotikapi.schemes.fields import (
    id_field,
    comment_field,
    routing_mark_field,
    bytes_field,
    log_prefix_field,
    field_gen,
)


class NatScheme(ToAddresses, SrcAddressBlock, ActionBlock, BaseModel):
    id: str = id_field
    bytes: int = bytes_field
    comment: str = comment_field
    disabled: bool = Field(default=False)
    to_ports: int = field_gen("to_ports", default=None)
    dst_port: int = field_gen("dst_port", default=None)
    dynamic: bool = Field(default=False, frozen=True, exclude=True)
    invalid: bool = Field(default=False, frozen=True, exclude=True)
    log: bool = Field(default=False)
    log_prefix: str = log_prefix_field
    packets: int = Field(default=0, frozen=True, exclude=True)
    protocol: str = Field(default="protocol")
    routing_mark: str = routing_mark_field
    in_interface: str = field_gen("in_interface")
    out_interface: str = field_gen("out_interface")

    @staticmethod
    def api_patch(_id=None):
        if _id:
            return f"/rest/ip/firewall/nat/{_id}"
        return "/rest/ip/firewall/nat"

    model_config = ConfigDict(populate_by_name=True)
